'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var compiler = require('svelte/compiler');
var Markdown = require('markdown-it');
var markdownItAnchor = require('markdown-it-anchor');
var prettier = require('prettier');
var Prism = require('prismjs');
require('prismjs/components/prism-bash');
require('prism-svelte');
var isRelativeUrl = require('is-relative-url');
var resolve = require('@rollup/plugin-node-resolve');
var rollupPluginSvelte = require('rollup-plugin-svelte');
var virtual = require('@rollup/plugin-virtual');
var rollupPluginTerser = require('rollup-plugin-terser');
var fs = require('fs-extra');
var path = require('path');
var crypto = require('crypto');
var htmlminifier = require('html-minifier');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var Markdown__default = /*#__PURE__*/_interopDefaultLegacy(Markdown);
var markdownItAnchor__default = /*#__PURE__*/_interopDefaultLegacy(markdownItAnchor);
var prettier__default = /*#__PURE__*/_interopDefaultLegacy(prettier);
var Prism__default = /*#__PURE__*/_interopDefaultLegacy(Prism);
var isRelativeUrl__default = /*#__PURE__*/_interopDefaultLegacy(isRelativeUrl);
var resolve__default = /*#__PURE__*/_interopDefaultLegacy(resolve);
var rollupPluginSvelte__default = /*#__PURE__*/_interopDefaultLegacy(rollupPluginSvelte);
var virtual__default = /*#__PURE__*/_interopDefaultLegacy(virtual);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var htmlminifier__default = /*#__PURE__*/_interopDefaultLegacy(htmlminifier);

const aliases = {
  sh: "bash",
  js: "javascript",
};

let md;

function preprocessReadme(opts) {
  const prefixUrl = opts.prefixUrl || `${opts.homepage}/tree/master/`;

  if (!md) {
    md = new Markdown__default['default']({
      html: true,
      linkify: true,
      typographer: true,
      highlight(source, lang) {
        if (lang === "svelte") {
          const regex = new RegExp('"' + opts.name + '"', "g");
          const modifiedSource = encodeURI(source.replace(regex, '"' + opts.svelte + '"'));
          const formattedCode = prettier__default['default'].format(source, {
            parser: "svelte",
            // @ts-ignore
            svelteBracketNewLine: true,
          });
          const svelteCode = Prism__default['default'].highlight(formattedCode, Prism__default['default'].languages.svelte, "svelte");
          return `<pre class="language-${lang}" data-svelte="${modifiedSource}">{@html \`${svelteCode}\`}</pre>`;
        }

        try {
          const alias_lang = aliases[lang] || lang;
          return `<pre class="language-${alias_lang}">{@html \`${Prism__default['default'].highlight(
            source,
            Prism__default['default'].languages[alias_lang],
            alias_lang
          )}\`}</pre>`;
        } catch (e) {
          console.error(`Could not highlight language "${lang}".`);
          return "";
        }
      },
    });

    md.use(markdownItAnchor__default['default']);
  }

  return {
    markup: ({ content, filename }) => {
      if (/node_modules/.test(filename) || !filename.endsWith(".md")) return null;
      let script_content = "";
      let style_content = "";
      let result = md.render(content);
      let cursor = 0;

      const ast = compiler.parse(result);

      compiler.walk(ast, {
        enter(node, parent) {
          if (node.type === "Attribute" && node.name === "href") {
            const value = node.value[0];

            if (value && !value.raw.startsWith("#") && isRelativeUrl__default['default'](value.raw)) {
              const relative_path = new URL(value.raw, prefixUrl).href;
              result = result.replace(value.raw, relative_path);
              cursor += relative_path.length - value.raw.length;
            }
          }

          if (node.type === "Style") {
            style_content += result.slice(node.content.start, node.content.end);
            const replace_style = result.slice(node.start + cursor, node.end + cursor);
            result = result.replace(replace_style, "");
            cursor -= replace_style.length;
          }

          if (node.type === "Attribute" && node.name === "data-svelte") {
            const raw_value = node.value[0].raw;
            const value = decodeURI(raw_value);
            const value_ast = compiler.parse(value);
            const markup =
              `<div class="code-fence">` + value.slice(value_ast.html.start, value_ast.html.end) + "</div>";
            const replace = result.slice(parent.start + cursor, parent.end + cursor);
            result = result.replace(replace, markup + replace.replace(raw_value, ""));
            cursor += markup.length - raw_value.length;

            compiler.walk(value_ast, {
              enter(node) {
                if (node.type === "Script") {
                  script_content += value.slice(node.content.start, node.content.end);
                }
              },
            });
          }
        },
      });

      return {
        code: `
            <script>${script_content}</script>
            <style>${style_content}</style>
            <main class="markdown-body">${result}</main>`,
      };
    },
  };
}

const css = `.anchor {
  float: left;
  line-height: 1;
  margin-left: -20px;
  padding-right: 4px;
}

.anchor:focus {
  outline: none;
}

h1:hover .anchor,
h2:hover .anchor,
h3:hover .anchor,
h4:hover .anchor,
h5:hover .anchor,
h6:hover .anchor {
  text-decoration: none;
}main {
  -ms-text-size-adjust: 100%;
  -webkit-text-size-adjust: 100%;
  line-height: 1.5;
  color: #24292e;
  font-family: -apple-system,BlinkMacSystemFont,Segoe UI,Helvetica,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji;
  font-size: 16px;
  line-height: 1.5;
  word-wrap: break-word;
}

details {
  display: block;
}

summary {
  display: list-item;
}

a {
  background-color: initial;
}

a:active,
a:hover {
  outline-width: 0;
}

strong {
  font-weight: inherit;
  font-weight: bolder;
}

h1 {
  font-size: 2em;
  margin: .67em 0;
}

img {
  border-style: none;
}

code,
kbd,
pre {
  font-family: monospace,monospace;
  font-size: 1em;
}

hr {
  box-sizing: initial;
  height: 0;
  overflow: visible;
}

input {
  font: inherit;
  margin: 0;
}

input {
  overflow: visible;
}

[type=checkbox] {
  box-sizing: border-box;
  padding: 0;
}

* {
  box-sizing: border-box;
}

input {
  font-family: inherit;
  font-size: inherit;
  line-height: inherit;
}

a {
  color: #0366d6;
  text-decoration: none;
}

a:hover {
  text-decoration: underline;
}

strong {
  font-weight: 600;
}

hr {
  height: 0;
  margin: 15px 0;
  overflow: hidden;
  background: transparent;
  border: 0;
  border-bottom: 1px solid #dfe2e5;
}

hr:after,
hr:before {
  display: table;
  content: "";
}

hr:after {
  clear: both;
}

table {
  border-spacing: 0;
  border-collapse: collapse;
}

td,
th {
  padding: 0;
}

details summary {
  cursor: pointer;
}

kbd {
  display: inline-block;
  padding: 3px 5px;
  font: 11px SFMono-Regular,Consolas,Liberation Mono,Menlo,monospace;
  line-height: 10px;
  color: #444d56;
  vertical-align: middle;
  background-color: #fafbfc;
  border: 1px solid #d1d5da;
  border-radius: 3px;
  box-shadow: inset 0 -1px 0 #d1d5da;
}

h1,
h2,
h3,
h4,
h5,
h6 {
  margin-top: 0;
  margin-bottom: 0;
}

h1 {
  font-size: 32px;
}

h1,
h2 {
  font-weight: 600;
}

h2 {
  font-size: 24px;
}

h3 {
  font-size: 20px;
}

h3,
h4 {
  font-weight: 600;
}

h4 {
  font-size: 16px;
}

h5 {
  font-size: 14px;
}

h5,
h6 {
  font-weight: 600;
}

h6 {
  font-size: 12px;
}

p {
  margin-top: 0;
  margin-bottom: 10px;
}

blockquote {
  margin: 0;
}

ol,
ul {
  padding-left: 0;
  margin-top: 0;
  margin-bottom: 0;
}

ol ol,
ul ol {
  list-style-type: lower-roman;
}

ol ol ol,
ol ul ol,
ul ol ol,
ul ul ol {
  list-style-type: lower-alpha;
}

dd {
  margin-left: 0;
}

code,
pre {
  font-family: SFMono-Regular,Consolas,Liberation Mono,Menlo,monospace;
  font-size: 12px;
}

pre {
  margin-top: 0;
  margin-bottom: 0;
}

input::-webkit-inner-spin-button,
input::-webkit-outer-spin-button {
  margin: 0;
  -webkit-appearance: none;
  appearance: none;
}

:checked+.radio-label {
  position: relative;
  z-index: 1;
  border-color: #0366d6;
}

hr {
  border-bottom-color: #eee;
}

kbd {
  display: inline-block;
  padding: 3px 5px;
  font: 11px SFMono-Regular,Consolas,Liberation Mono,Menlo,monospace;
  line-height: 10px;
  color: #444d56;
  vertical-align: middle;
  background-color: #fafbfc;
  border: 1px solid #d1d5da;
  border-radius: 3px;
  box-shadow: inset 0 -1px 0 #d1d5da;
}

main:after,
.markdown-body:before {
  display: table;
  content: "";
}

main:after {
  clear: both;
}

main>:first-child {
  margin-top: 0!important;
}

main>:last-child {
  margin-bottom: 0!important;
}

a:not([href]) {
  color: inherit;
  text-decoration: none;
}

blockquote,
details,
dl,
ol,
p,
pre,
table,
ul {
  margin-top: 0;
  margin-bottom: 16px;
}

hr {
  height: .25em;
  padding: 0;
  margin: 24px 0;
  background-color: #e1e4e8;
  border: 0;
}

blockquote {
  padding: 0 1em;
  color: #6a737d;
  border-left: .25em solid #dfe2e5;
}

blockquote>:first-child {
  margin-top: 0;
}

blockquote>:last-child {
  margin-bottom: 0;
}

h1,
h2,
h3,
h4,
h5,
h6 {
  margin-top: 24px;
  margin-bottom: 16px;
  font-weight: 600;
  line-height: 1.25;
}

h1 {
  font-size: 2em;
}

h1,
h2 {
  padding-bottom: .3em;
  border-bottom: 1px solid #eaecef;
}

h2 {
  font-size: 1.5em;
}

h3 {
  font-size: 1.25em;
}

h4 {
  font-size: 1em;
}

h5 {
  font-size: .875em;
}

h6 {
  font-size: .85em;
  color: #6a737d;
}

ol,
ul {
  padding-left: 2em;
}

ol ol,
ol ul,
ul ol,
ul ul {
  margin-top: 0;
  margin-bottom: 0;
}

li {
  word-wrap: break-all;
}

li>p {
  margin-top: 16px;
}

li+li {
  margin-top: .25em;
}

dl {
  padding: 0;
}

dl dt {
  padding: 0;
  margin-top: 16px;
  font-size: 1em;
  font-style: italic;
  font-weight: 600;
}

dl dd {
  padding: 0 16px;
  margin-bottom: 16px;
}

table {
  display: block;
  width: 100%;
  overflow: auto;
}

table th {
  font-weight: 600;
}

table td,
table th {
  padding: 6px 13px;
  border: 1px solid #dfe2e5;
}

table tr {
  background-color: #fff;
  border-top: 1px solid #c6cbd1;
}

table tr:nth-child(2n) {
  background-color: #f6f8fa;
}

img {
  max-width: 100%;
  box-sizing: initial;
  background-color: #fff;
}

img[align=right] {
  padding-left: 20px;
}

img[align=left] {
  padding-right: 20px;
}

code {
  padding: .2em .4em;
  margin: 0;
  font-size: 85%;
  background-color: rgba(27,31,35,.05);
  border-radius: 3px;
}

pre {
  word-wrap: normal;
}

pre>code {
  padding: 0;
  margin: 0;
  font-size: 100%;
  word-break: normal;
  white-space: pre;
  background: transparent;
  border: 0;
}

.highlight {
  margin-bottom: 16px;
}

.highlight pre {
  margin-bottom: 0;
  word-break: normal;
}

.highlight pre,
pre {
  padding: 16px;
  overflow: auto;
  font-size: 85%;
  line-height: 1.45;
  background-color: #f6f8fa;
  border-radius: 3px;
}

pre code {
  display: inline;
  max-width: auto;
  padding: 0;
  margin: 0;
  overflow: visible;
  line-height: inherit;
  word-wrap: normal;
  background-color: initial;
  border: 0;
}
`;

function hashREADME() {
  try {
    const path_readme = path__default['default'].join(process.cwd(), "README.md");
    const readme = fs__default['default'].readFileSync(path_readme);
    return "." + crypto.createHash("md5").update(readme).digest("hex").slice(0, 8);
  } catch (error) {
    console.log(error);
    process.exit(1);
  }
}

function getPackageJSON() {
  try {
    const path_pkg = path__default['default'].join(process.cwd(), "package.json");
    const pkg = JSON.parse(fs__default['default'].readFileSync(path_pkg, "utf-8"));
    if (!pkg.name) throw Error(`Package name is required as "name".`);
    if (!pkg.svelte) throw Error(`Svelte code entry is required as "svelte".`);

    return {
      name: pkg.name,
      svelte: pkg.svelte,
      description: pkg.description,
      homepage: pkg.homepage,
    };
  } catch (error) {
    console.log(error);
    process.exit(1);
  }
}

const custom_css = `
  .token.language-javascript { color: #24292e; }
  .token.language-javascript .function { color: #005cc5; }
  .token.language-javascript .string { color: #032f62; }
  .token.language-javascript .number { color: #005cc5; }
  .token.language-javascript .keyword { color: #d73a49; }
  .token.each { color: #d73a49; }
  .token.punctuation { color: #24292e }
  .token.tag { color: #22863a; }
  .token.attr-name { color: #6f42c1; }
  .token.attr-value { color: #032f62; }
  .token.operator { color: #d73a49; }
  .token.comment { color: #6a737d; }
  .token.function { color: #6f42c1; }
  .token.string { color: #032f62; }

  .token.each .language-javascript:last-child { color: #e36209; }

  .language-css { color: #032f62; }
  .language-css .selector { color: #22863a; }
  .language-css .property { color: #005cc5; }

  .code-fence { padding: 24px 15px; border: 1px solid #eaecef; border-bottom: 0; }

  main {
    box-sizing: border-box;
    max-width: 980px;
    margin: 0 auto;
    padding: 45px;
  }

  @media (max-width: 767px) {
    main { padding: 15px; }
  }
`;

/**
 * createConfig
 * @param {Object} opts
 * @param {boolean} opts.minify - set to `true` to minify the HTML/JS
 * @param {string} opts.outDir - set the folder to emit the files
 * @param {string} opts.style - custom CSS appended to the <style> block
 * @param {boolean} opts.disableDefaultCSS - set to `true` to omit the default GitHub styles
 * @param {string} opts.prefixUrl - Value to prepend to relative URLs (i.e. GitHub repo URL)
 * @param {object} opts.svelte - `rollup-plugin-svelte` options
 * @param {Array} opts.plugins - Rollup plugins
 * @param {object} opts.output - Rollup output options
 */
function createConfig(opts) {
  const minify = opts.minify === true;
  const pkg = getPackageJSON();
  const hash = minify ? hashREADME() : "";
  const output_dir = opts.outDir || "dist";

  const template = `
  <!DOCTYPE html>
  <html lang="en">
    <head>
      <meta charset="utf-8" />
      <meta name="viewport" content="width=device-width, initial-scale=1.0" />
      <meta name="description" content="${pkg.description || `${pkg.name} demo`}" />
      <title>${pkg.name}</title>
      <style>
        ${!opts.disableDefaultCSS ? css : ""}
        ${custom_css}
        ${opts.style || ""}
      </style>
    </head>
    <body>
      <noscript>You need to enable JavaScript to run this app.</noscript>
      <script src="s${hash}.js"></script>
    </body>
  </html>
`;

  if (minify) fs__default['default'].removeSync(output_dir);
  fs__default['default'].ensureFileSync(`${output_dir}/index.html`);
  fs__default['default'].writeFileSync(
    `${output_dir}/index.html`,
    minify
      ? htmlminifier__default['default'].minify(template, {
          collapseWhitespace: true,
          conservativeCollapse: true,
          minifyCSS: true,
          removeEmptyAttributes: true,
        })
      : template
  );

  return {
    watch: { clearScreen: false },
    input: "entry",
    output: {
      format: "iife",
      name: "app",
      file: `${output_dir}/s${hash}.js`,
      ...(opts.output || {}),
    },
    plugins: [
      virtual__default['default']({
        entry: `import App from "./README.md";
                const app = new App({ target: document.body });
                export default app;`,
      }),
      rollupPluginSvelte__default['default']({
        ...(opts.svelte || {}),
        extensions: [".svelte", ".md"],
        preprocess: [preprocessReadme({ ...pkg, prefixUrl: opts.prefixUrl })],
      }),
      resolve__default['default'](),
      ...(opts.plugins || []),
      minify && rollupPluginTerser.terser(),
    ].filter(Boolean),
  };
}

exports.default = createConfig;
exports.preprocessReadme = preprocessReadme;
